"""
A Pygments_ style based on the dark background variant of Solarized_.
.. _Pygments: http://pygments.org/
.. _Solarized: http://ethanschoonover.com/solarized
"""

from pygments.style import Style
from pygments.token import Token, Comment, Name, Keyword, Generic, Number, Operator, String

BASE03 = '#002B36'
BASE02 = '#073642'
BASE01 = '#586E75'
BASE00 = '#657B83'
BASE0 = '#839496'
BASE1 = '#93A1A1'
BASE2 = '#EEE8D5'
BASE3 = '#FDF6E3'
YELLOW = '#B58900'
ORANGE = '#CB4B16'
RED = '#DC322F'
MAGENTA = '#D33682'
VIOLET = '#6C71C4'
BLUE = '#268BD2'
CYAN = '#2AA198'
GREEN = '#859900'


class SolarizedStyle(Style):
    background_color = BASE03
    styles = {
        Keyword: GREEN,
        Keyword.Constant: ORANGE,
        Keyword.Declaration: BLUE,
        # Keyword.Namespace
        # Keyword.Pseudo
        Keyword.Reserved: BLUE,
        Keyword.Type: RED,

        # Name
        Name.Attribute: BASE1,
        Name.Builtin: YELLOW,
        Name.Builtin.Pseudo: BLUE,
        Name.Class: BLUE,
        Name.Constant: ORANGE,
        Name.Decorator: BLUE,
        Name.Entity: ORANGE,
        Name.Exception: ORANGE,
        Name.Function: BLUE,
        # Name.Label
        # Name.Namespace
        # Name.Other
        Name.Tag: BLUE,
        Name.Variable: BLUE,
        # Name.Variable.Class
        # Name.Variable.Global
        # Name.Variable.Instance

        # Literal
        # Literal.Date
        String: CYAN,
        String.Backtick: BASE01,
        String.Char: CYAN,
        String.Doc: BASE1,
        # String.Double
        String.Escape: ORANGE,
        String.Heredoc: BASE1,
        # String.Interpol
        # String.Other
        String.Regex: RED,
        # String.Single
        # String.Symbol
        Number: CYAN,
        # Number.Float
        # Number.Hex
        # Number.Integer
        # Number.Integer.Long
        # Number.Oct

        Operator: GREEN,
        # Operator.Word

        # Punctuation: ORANGE,

        Comment: BASE01,
        # Comment.Multiline
        Comment.Preproc: GREEN,
        # Comment.Single
        Comment.Special: GREEN,

        # Generic
        Generic.Deleted: CYAN,
        Generic.Emph: 'italic',
        Generic.Error: RED,
        Generic.Heading: ORANGE,
        Generic.Inserted: GREEN,
        # Generic.Output
        # Generic.Prompt
        Generic.Strong: 'bold',
        Generic.Subheading: ORANGE,
        # Generic.Traceback

        Token: BASE1,
        Token.Other: ORANGE,
    }



#!/usr/bin/env python3
"""
:copyright: (c) 2018 Pinn Technologies, Inc.
:license: All rights reserved
"""

import click
import _thread as thread
from .connection import Connection
from .app import app, command_processor


class WSH:

    def __init__(self, host, commands=None):
        self.host = host
        if commands:
            for key, value in commands.items():
                command_processor.register_command(key, value)

    def run(self):
        def connection():
            Connection(self.host, app)
        thread.start_new_thread(connection, ())
        app.run()


@click.command()
@click.argument('host')
def wsh(host):
    WSH(host).run()


if __name__ == '__main__':
    wsh()



"""
:copyright: (c) 2018 Pinn Technologies, Inc.
:license: All rights reserved
"""
from ._version import get_versions
__version__ = get_versions()['version']
del get_versions

from .wsh import WSH  # NOQA



"""
:copyright: (c) 2018 Pinn Technologies, Inc.
:license: All rights reserved
"""

import json
import click
import websocket
from blinker import signal
from prompt_toolkit.document import Document


class Connection:

    def __init__(self, ws_url, app, debug=True):
        signal('wsh-send').connect(self.send)
        signal('wsh-close').connect(self.close)
        self.ws_url = ws_url
        self.app = app
        self.output = app.layout.container.children[0].content.buffer
        self.ws = websocket.WebSocketApp(ws_url,
                                         on_message=self.on_message,
                                         on_error=self.on_error,
                                         on_close=self.on_close,
                                         on_open=self.on_open)
        self.ws.run_forever(ping_interval=30)

    def display(self, string, direction=None):
        """Outputs the given string to the UI."""
        if direction is None:
            cursor = ''
        elif direction == 'in':
            cursor = '<<< '
        elif direction == 'out':
            cursor = '>>> '
        output = self.output.text + u'\n' + cursor + string
        self.output.document = Document(text=output, cursor_position=len(output))

    def info(self, string):
        """Output given string as info, which is a right justified text element."""
        width = click.get_terminal_size()[0]
        output = string.rjust(width)
        output = self.output.text + '\n' + output
        self.output.document = Document(text=output, cursor_position=len(output))

    def send(self, sender, **kw):
        """Send a data payload to the server."""
        data = kw['data']
        self.ws.send(data)
        try:
            json_data = json.loads(data)
            self.display('sent', direction='out')
            self.display(json.dumps(json_data, indent=2))
        except Exception:
            self.display('sent', direction='out')
            self.display(data)

    def close(self, sender, **kw):
        """Close connection to the server."""
        self.ws.close()

    def on_message(self, ws, message):
        try:
            data = json.loads(message)
            self.display('received', direction='in')
            self.display(json.dumps(data, indent=2))
        except Exception:
            self.display('received', direction='in')
            self.display(message.decode('utf-8'))

    def on_error(self, ws, error):
        message = "Error occured during connection to ({})".format(self.ws_url)
        self.info(message)
        self.info(str(error))

    def on_close(self, ws):
        self.info('Connection closed with ({})'.format(self.ws_url))

    def on_open(self, ws):
        self.info('Connection opened with ({})'.format(self.ws_url))
